@@ -1,30 +1,40 @@
 /*
- * TransFig: Facility for Translating Fig code
+ * Fig2dev: Translate Fig code to various Devices
  * Copyright (c) 1991 by Micah Beck
  * Parts Copyright (c) 1985-1988 by Supoj Sutanthavibul
  * Parts Copyright (c) 1989-2002 by Brian V. Smith
+ * Parts Copyright (c) 2015-2017 by Thomas Loimer
  *
  * Any party obtaining a copy of these files is granted, free of charge, a
  * full and unrestricted irrevocable, world-wide, paid up, royalty-free,
- * nonexclusive right and license to deal in this software and
- * documentation files (the "Software"), including without limitation the
- * rights to use, copy, modify, merge, publish and/or distribute copies of
- * the Software, and to permit persons who receive copies from any such
- * party to do so, with the only requirement being that this copyright
- * notice remain intact.
+ * nonexclusive right and license to deal in this software and documentation
+ * files (the "Software"), including without limitation the rights to use,
+ * copy, modify, merge, publish, distribute, sublicense and/or sell copies
+ * of the Software, and to permit persons who receive copies from any such
+ * party to do so, with the only requirement being that the above copyright
+ * and this permission notice remain intact.
  *
  */
 
 /*
- *	genbox.c: Empty box driver for fig2dev translator
+ * genbox.c: convert fig to empty box, for inclusion by LaTeX
  *
 */
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+
+#include <stdio.h>
+#include <stdlib.h>
+
 #include "fig2dev.h"
-#include "object.h"
+#include "object.h"	/* does #include <X11/xpm.h> */
 
 void
 genbox_option(char opt, char *optarg)
 {
+	(void)	optarg;
+
 	switch (opt) {
 
 	case 'G':
@@ -40,6 +50,10 @@ genbox_option(char opt, char *optarg)
 void
 genbox_start(F_compound *objects)
 {
+	/* print any whole-figure comments prefixed with "%" */
+	if (objects->comments)
+		print_comments("% ",objects->comments, "");
+
 	/* draw box */
 	fprintf(tfp, "\\makebox[%.3fin]{\\rule{0in}{%.3fin}}\n",
 		(urx-llx)*mag/ppi, (ury-lly)*mag/ppi);
@@ -55,7 +69,7 @@ genbox_end(void)
 struct driver dev_box = {
 	genbox_option,
 	genbox_start,
-	gendev_nogrid,
+	(void(*)(float,float))gendev_null,
 	gendev_null,
 	gendev_null,
 	gendev_null,
