@@ -1,35 +1,50 @@
 /*
- * TransFig: Facility for Translating Fig code
+ * Fig2dev: Translate Fig code to various Devices
  * Copyright (c) 1991 by Micah Beck
  * Parts Copyright (c) 1985-1988 by Supoj Sutanthavibul
- * Parts Copyright (c) 1989-2002 by Brian V. Smith
+ * Parts Copyright (c) 1989-2015 by Brian V. Smith
+ * Parts Copyright (c) 2015-2019 by Thomas Loimer
  *
  * Any party obtaining a copy of these files is granted, free of charge, a
  * full and unrestricted irrevocable, world-wide, paid up, royalty-free,
- * nonexclusive right and license to deal in this software and
- * documentation files (the "Software"), including without limitation the
- * rights to use, copy, modify, merge, publish and/or distribute copies of
- * the Software, and to permit persons who receive copies from any such
- * party to do so, with the only requirement being that this copyright
- * notice remain intact.
+ * nonexclusive right and license to deal in this software and documentation
+ * files (the "Software"), including without limitation the rights to use,
+ * copy, modify, merge, publish, distribute, sublicense and/or sell copies
+ * of the Software, and to permit persons who receive copies from any such
+ * party to do so, with the only requirement being that the above copyright
+ * and this permission notice remain intact.
  *
  */
 
 /*
- *	genpictex.C : PiCTeX driver for fig2dev
+ * genpictex.C: convert fig to PiCTeX
  *
- *	Author Micah Beck, Cornell University, 4/88
- *    Color, rotated text and ISO-chars added by Herbert Bauer 11/91
-*/
+ * Author Micah Beck, Cornell University, 4/88
+ *	Color, rotated text and ISO-chars added by Herbert Bauer 11/91
+ */
 
-#include "fig2dev.h"
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+
+#include <stdio.h>
+#include <stdlib.h>
+#include <string.h>
+#ifdef HAVE_UNISTD_H
+#include <unistd.h>
+#endif
+#include <math.h>
 #ifdef HAVE_GETPWUID
 #include <pwd.h>
 #endif
-#include "object.h"
+
+#include "pi.h"
+#include "fig2dev.h"	/* includes "bool.h" */
+#include "object.h"	/* does #include <X11/xpm.h> */
 #include "genlatex.h"
 #include "setfigfont.h"
 #include "texfonts.h"
+#include "psfonts.h"
 #include "localmath.h"
 #include "creationdate.h"
 
@@ -381,22 +396,22 @@ put_box(int llx, int lly, int urx, int ury, F_line *l)
 		    ((convy(lly/ppi)+convy(ury/ppi))/2)*CONVUNIT);
 	       fprintf(tfp,"\\linethickness=%dpt\n", l->thickness);
 	   }
-	   else if (l->fill_style != UNFILLED && l->fill_style > (int)(BLACK_FILL*3/4))
+	   else if (l->fill_style > BLACK_FILL*3/4)
 	   {
 	       fprintf(tfp,"\\setshadegrid span <1pt>\n");
 	       fprintf(tfp,"\\shaderectangleson\n");
 	   }
-	   else if (l->fill_style != UNFILLED && l->fill_style > (int)(BLACK_FILL*2/4))
+	   else if (l->fill_style > BLACK_FILL*2/4)
 	   {
 	       fprintf(tfp,"\\setshadegrid span <2pt>\n");
 	       fprintf(tfp,"\\shaderectangleson\n");
 	   }
-	   else if (l->fill_style != UNFILLED && l->fill_style > (int)(BLACK_FILL*1/4))
+	   else if (l->fill_style > BLACK_FILL*1/4)
 	   {
 	       fprintf(tfp,"\\setshadegrid span <4pt>\n");
 	       fprintf(tfp,"\\shaderectangleson\n");
 	   }
-	   else if (l->fill_style && l->fill_style == WHITE_FILL)
+	   else if (l->fill_style == WHITE_FILL)
 	   {
 	       fprintf(stderr,"WHITE_FILL not implemented for boxes\n");
 	   }
@@ -633,7 +648,7 @@ genpictex_text(F_text *t)
 	  baselineskip = (texsize * 1.2);
 
 #ifdef NFSS
-	  fprintf(tfp, "\\put{\\SetFigFontNFSS{%d}{%.1f}{%s}{%s}{%s}",
+	  fprintf(tfp, "\\put{\\SetFigFont{%d}{%.1f}{%s}{%s}{%s}",
 				 texsize, baselineskip,
 				 TEXFAMILY(t->font),TEXSERIES(t->font),TEXSHAPE(t->font));
 #else
@@ -664,7 +679,6 @@ genpictex_text(F_text *t)
 	else
 		for(cp = (unsigned char*)t->cstring; *cp; cp++) {
 #ifdef I18N
-		    extern bool support_i18n;
 		    if (support_i18n && (t->font <= 2))
 			    fputc(*cp, tfp);
 		    else
