@@ -1,41 +1,57 @@
 /*
- * TransFig: Facility for Translating Fig code
- * Copyright (c) 2002 by Christian Gollwitzer (auriocus)
- * Parts Copyright (c) 1999 by T. Sato
- * Parts Copyright (c) 1989-1999 by Brian V. Smith
+ * Fig2dev: Translate Fig code to various Devices
+ * Copyright (c) 1991 by Micah Beck
+ * Parts Copyright (c) 1985-1988 by Supoj Sutanthavibul
+ * Parts Copyright (c) 1989-2015 by Brian V. Smith
+ * Parts Copyright (c) 2015-2019 by Thomas Loimer
  *
  * Any party obtaining a copy of these files is granted, free of charge, a
  * full and unrestricted irrevocable, world-wide, paid up, royalty-free,
- * nonexclusive right and license to deal in this software and
- * documentation files (the "Software"), including without limitation the
- * rights to use, copy, modify, merge, publish and/or distribute copies of
- * the Software, and to permit persons who receive copies from any such
- * party to do so, with the only requirement being that this copyright
- * notice remain intact.
+ * nonexclusive right and license to deal in this software and documentation
+ * files (the "Software"), including without limitation the rights to use,
+ * copy, modify, merge, publish, distribute, sublicense and/or sell copies
+ * of the Software, and to permit persons who receive copies from any such
+ * party to do so, with the only requirement being that the above copyright
+ * and this permission notice remain intact.
  *
  */
 
-/* genshape.c: LaTeX shapepar driver for fig2dev
+/*
+ * genshape.c: convert fig to LaTeX shapepar
+ *
+ * Copyright (c) 2002 by Christian Gollwitzer (auriocus)
  * derived from genmap.c
  *
+ * genmap.c:
+ * Copyright (c) 1999 by T. Sato
+ *
  * "Christian Gollwitzer" <auriocus@web.de>
- * if an object has a comment starting with '+', it becomes
- * a filled shape. If the comment starts with '-'
- * then it is a hole. The objects are composed
- * in the order of their depths. A word following the initial
- * '+' or '-' is considered to be an identifier that groups
- * different shapes to one text segment. The order of filling these
- * segments is determined by the alphabetic sort order of the identifiers.
- * The special comments "center", "width" and "height" define
- * position and dimension.
+ * If an object has a comment starting with '+', it becomes a filled shape.
+ * If the comment starts with '-' then it is a hole. The objects are
+ * composed in the order of their depths. A word following the initial
+ * '+' or '-' is considered to be an identifier that groups different shapes
+ * to one text segment. The order of filling these segments is determined by
+ * the alphabetic sort order of the identifiers. The special comments
+ * "center", "width" and "height" define position and dimension.
  *
  */
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+
+#include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#ifdef HAVE_UNISTD_H
+#include <unistd.h>
+#endif
+#include <math.h>
 #include <ctype.h>
-#include "fig2dev.h"
-#include "object.h"
+
+#include "pi.h"
+#include "fig2dev.h"	/* includes "bool.h" */
+#include "object.h"	/* does #include <X11/xpm.h> */
 
 static void die(const char * msg) {
  fprintf(stderr, "fig2dev(shape): %s\n", msg);
@@ -163,6 +179,7 @@ static void add_shapegroup(int shapestart, int shapeend) {
 }
 
 /** debugging stuff **/
+/*
 static void
 print_shapegroups(void) {
   int i;
@@ -171,6 +188,7 @@ print_shapegroups(void) {
       i, shapegroups[i].shapestart, shapegroups[i].shapeend,
       shapes[shapegroups[i].shapestart].groupname);
 }
+*/
 
 #define fscale 10
 
@@ -1007,7 +1025,6 @@ genshape_end(void)
   int	 num_above_borders, num_below_borders;
   int	 snr, aind, bind;
   bool	 equal;
-  char	*oldgroupname;
   int	 oldscanlines=0;
 
 
@@ -1029,8 +1046,6 @@ genshape_end(void)
       points[i].shapenr=shapenr;
   }
 
-
-  oldgroupname=shapes[0].groupname;
   /* now find the borders in between the shapegroups */
   shapestart=0;
   for (i=0; i<num_shapes; i++) {
@@ -1244,7 +1259,7 @@ genshape_end(void)
 	   else
 	     special_block(above_block_borders[aind],'j');
 
-	     aind+=2; /* skip both */
+	   aind+=2; /* skip both */
 	   continue;
 	 } else {
 	   /* difference more than a 0-gap */
@@ -1408,7 +1423,7 @@ struct driver dev_shape = {
 	genshape_arc,
 	genshape_ellipse,
 	genshape_line,
-	gendev_null,
+	(void (*)(F_spline *))gendev_null,
 	genshape_text,
 	genshape_end,
 	INCLUDE_TEXT
