@@ -1,34 +1,47 @@
 /*
- * TransFig: Facility for Translating Fig code
+ * Fig2dev: Translate Fig code to various Devices
  * Copyright (c) 1991 by Micah Beck
  * Parts Copyright (c) 1985-1988 by Supoj Sutanthavibul
- * Parts Copyright (c) 1989-2002 by Brian V. Smith
+ * Parts Copyright (c) 1989-2015 by Brian V. Smith
+ * Parts Copyright (c) 2015-2018 by Thomas Loimer
  *
  * Any party obtaining a copy of these files is granted, free of charge, a
  * full and unrestricted irrevocable, world-wide, paid up, royalty-free,
- * nonexclusive right and license to deal in this software and
- * documentation files (the "Software"), including without limitation the
- * rights to use, copy, modify, merge, publish and/or distribute copies of
- * the Software, and to permit persons who receive copies from any such
- * party to do so, with the only requirement being that this copyright
- * notice remain intact.
+ * nonexclusive right and license to deal in this software and documentation
+ * files (the "Software"), including without limitation the rights to use,
+ * copy, modify, merge, publish, distribute, sublicense and/or sell copies
+ * of the Software, and to permit persons who receive copies from any such
+ * party to do so, with the only requirement being that the above copyright
+ * and this permission notice remain intact.
  *
  */
 
 /*
- * gentikz.c: TeX/LaTeX tikz driver for fig2dev
- * Copyright (c) 2015 by Thomas Loimer
+ * gentikz.c: convert fig to tikz macro language for TeX/LaTeX
+ *
+ * Author: Thomas Loimer
  *
  * BUGS:	o shades or tints of the default color may be incorrect
  *		o arrows on arcs and on very short lines may be different
  *		  from arcs produced by the postscript driver
  */
 
-#include "fig2dev.h"
-#include "object.h"
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+
+#include <stdio.h>
+#include <stdlib.h>
+#include <string.h>
+#include <math.h>
+#include "pi.h"
+
+#include "fig2dev.h"	/* includes "bool.h" */
+#include "object.h"	/* does #include <X11/xpm.h> */
 #include "texfonts.h"		/* texfontnames[] */
 #include "bound.h"
 #include "gentikz.h"
+#include "psfonts.h"
 
 /* String arrays and structs */
 extern char	*ISO1toTeX[];	/* iso2tex.c */
@@ -237,6 +250,7 @@ static bool	removesuffix = false;
 static bool	usetexfonts = false;
 static bool	allspecial = false;
 static bool	pagemode = false;
+static bool	nofigscaling = false;
 static char	*prepend = NULL;
 static int	encoding = 1;
 static int	verbose = 0;
@@ -267,7 +281,6 @@ static struct options	alt_options;
 			/* THICK_SCALE is of type float, see fig2dev.h */
 /* #define round	(int)... see fig2dev.h */
 #define PUT_POINT(p)	fprintf(tfp,"(%d,%d)", XCOORD(p->x), YCOORD(p->y))
-#define	NOTEXISTS(obj)	obj->thickness <= 0 && obj->fill_style == UNFILLED
 
 #define	MITERJOIN	0
 #define	BUTTCAP		0
@@ -339,6 +352,10 @@ gentikz_option(char opt, char *optarg)
 	    verbose = 1;		/* verbose mode */
 	    break;
 
+	case 'W':
+	    nofigscaling = true;
+	    break;
+
 	case 'w':		/* remove suffix from included graphics file */
 	    removesuffix = true;
 	    break;
@@ -651,7 +668,6 @@ define_arrow(int ttype, int indx)
 void
 gentikz_start(F_compound *objects)
 {
-	char	buf[20];    /* when changing 20, change also the sprintf() below! */
 	int	i;
 	double	splength;   /* unitlength expressed in TeX scaled points */
 
@@ -671,7 +687,7 @@ gentikz_start(F_compound *objects)
 	/* print any whole-figure comments prefixed with "%" */
 	if (objects->comments) {
 	    fprintf(tfp,"%%\n");
-	    print_comments("% ",objects->comments, "");
+	    print_comments("% ", objects->comments, "");
 	    fprintf(tfp,"%%\n");
 	}
 	if (pagemode) {
@@ -697,6 +713,8 @@ gentikz_start(F_compound *objects)
 	    if (pats_used)
 		fputs("\\usetikzlibrary{patterns}\n", tfp);
 	    fputs("\\parindent0pt\n\\begin{document}\n", tfp);
+	}
+	if (pagemode || nofigscaling) {
 /*	if (metric)	fprintf(tfp, "%% 4143.7 sp = (1/472.44) cm\n");
  *	else		fprintf(tfp, "%% 3946.9 sp = (1/1200) in\n");	*/
 	    fprintf(tfp, "\\tikzpicture[x=+%lisp, y=+%lisp]\n",
@@ -715,7 +733,10 @@ gentikz_start(F_compound *objects)
 	    fputs("  \\else\\dimen1\\dimen3\\fi", tfp);
 	    fputs("\\else\\ifdim\\dimen3=0pt\\dimen3\\dimen1\\fi\\fi\n", tfp);
 	    fputs("\\tikzpicture[x=+\\dimen1, y=+\\dimen3]\n", tfp);
-	    fprintf(tfp, "\\newdimen\\XFigu\\XFigu%lisp\n", (long) splength);
+	    fputs("{\\ifx\\XFigu\\undefined\\catcode`\\@11\n", tfp);
+	    fputs("\\def\\temp{\\alloc@1\\dimen\\dimendef\\insc@unt}", tfp);
+	    fputs("\\temp\\XFigu\\catcode`\\@12\\fi}\n", tfp);
+	    fprintf(tfp, "\\XFigu%lisp\n", (long) splength);
 	    fputs("% Uncomment to scale line thicknesses with the same\n", tfp);
 	    fputs("% factor as width of the drawing.\n", tfp);
 	    fputs("%\\pgfextractx\\XFigu{\\pgfqpointxy{1}{1}}\n", tfp);
@@ -860,15 +881,13 @@ put_picture(F_point *p, F_point *r, F_pic *pic)
 	 *					       |      |
 	 *					     2 +------+ 3
 	 */
-	int	n, dx, dy, rot;
+	int	n, dx, dy, rot = 0;
 	char	*c;
 
 	dx = r->x - p->x;
 	dy = r->y - p->y;
 	/* get the rotation, and write the height and width to dx and dy */
-	if (dx >= 0 && dy >= 0) {   /* >= 0 - to silence static analyzer warnings */
-	    rot = 0;
-	} else if (dx < 0 && dy < 0) {
+	if (dx < 0 && dy < 0) {
 	    dx = -dx;
 	    dy = -dy;
 	    rot = 180;
@@ -882,7 +901,7 @@ put_picture(F_point *p, F_point *r, F_pic *pic)
 	    dx = -dy;
 	    dy = rot;
 	    rot = 90;
-	}
+	} /* else dx >= 0 && dy >= 0: rot = 0 */
 
 	if (pic->flipped) {
 	    rot += 90;
@@ -895,7 +914,7 @@ put_picture(F_point *p, F_point *r, F_pic *pic)
 
 	if (removesuffix) {
 	    c = strrchr(pic->file,'.');
-	    n =  c == NULL ? strlen(pic->file) : c - pic->file;
+	    n =  c == NULL ? (int)strlen(pic->file) : c - pic->file;
 	} else {
 	    n = strlen(pic->file);
 	}
@@ -1151,8 +1170,6 @@ put_drawcmd(int style, int thickness, int pen_color, int fill_color,
 #define	FILLED	fill_style > UNFILLED && fill_style < NUMSHADES + NUMTINTS
 #define	PATTERN	fill_style >= NUMSHADES + NUMTINTS
 
-	int	n;
-
 	if (thickness > 0 ) {
 	    if (has_endcaps || (style != SOLID_LINE && style_val > 0.))
 		set_capstyle(cap_style);
@@ -1160,10 +1177,9 @@ put_drawcmd(int style, int thickness, int pen_color, int fill_color,
 	    set_width(thickness);
 	    set_stipple(style, style_val);
 
-	    if (FILLED || (PATTERN && fill_color == pen_color)) {
+	    if (FILLED) {
 		normalize(&fill_color, &fill_style);
-		if (fill_color == pen_color
-			&& (PATTERN || fill_style == NUMSHADES-1)) {
+		if (fill_color == pen_color && fill_style == NUMSHADES-1) {
 		    set_linefillcolor(pen_color);
 		} else {
 		    set_linecolor(pen_color);
@@ -1189,7 +1205,7 @@ put_drawcmd(int style, int thickness, int pen_color, int fill_color,
 
 	} else {	/* thickness <= 0 */
 
-	    if (FILLED || (PATTERN && fill_color == pen_color)) {
+	    if (FILLED) {
 		normalize(&fill_color, &fill_style);
 		set_fillcolor(fill_color, fill_style);
 		fputs("\\fill ", tfp);
@@ -1576,15 +1592,6 @@ gentikz_line(F_line *l)
 	/* print any comments prefixed with "%" */
 	print_comments("% ", l->comments, "");
 
-	/* all checks will be done in read.c */
-	/*	1 pt, 2pt && thickness > 0 - single point, line
-		> 3 pts, line, box, arc-box, polygon
-			PUT_DRAWCMD (eig PUT_DRAWORFILLCMD)
-			line - just put all points
-			box, arc-box, polygon - do not put the last point
-					      - verzögere um einen Punkt
-	*/
-
 	p = l->points;
 	q = p->next;
 
@@ -1625,7 +1632,8 @@ gentikz_line(F_line *l)
 	}
 
 	/* T_POLYLINE, T_BOX, T_POLYGON or T_ARC_BOX from here on below */
-	if (l->thickness <= 0 && l->fill_style == UNFILLED)
+	if (l->thickness <= 0 && l->fill_style == UNFILLED &&
+			!l->for_arrow && !l->back_arrow)
 	    return;
 
 	if (l->type == T_BOX || l->type == T_POLYGON || l->type == T_ARC_BOX) {
@@ -1653,6 +1661,7 @@ gentikz_line(F_line *l)
 void
 gentikz_spline(F_spline *s)
 {
+	print_comments("% ", s->comments, "");
 	fprintf(stderr, "Can't generate spline; omitting object\n");
 }
 
@@ -1662,9 +1671,9 @@ gentikz_ellipse(F_ellipse *e)
 	if (verbose)
 	    fputs("%%\n%% Fig ELLIPSE object\n%%\n", tfp);
 
-	print_comments("% ",e->comments, "");
+	print_comments("% ", e->comments, "");
 
-	if (NOTEXISTS(e)) return;
+	if (e->thickness <= 0 && e->fill_style == UNFILLED) return;
 
 	PUT_DRAWCMD(e, false, ROUNDCAP);
 
@@ -1728,7 +1737,7 @@ put_font(F_text *t)
 /*
  * Put the text, e.g.,
  *   \pgfsetfillcolor{blue}   % \pgftext obeys \pgfsetfillcolor
- *   \pgftext[base,left,at=\pgfqpointxy{3}{2},rotate=45] {Text here!};
+ *   \pgftext[base,left,at=\pgfqpointxy{3}{2},rotate=45] {Text here!}
  * Order ist important! First at=..., then rotate=... !
  */
 void
@@ -1740,7 +1749,7 @@ gentikz_text(F_text *t)
 	    fprintf(tfp, "%%\n%% Fig TEXT object\n%%\n");
 
 	/* print any comments prefixed with "%" */
-	print_comments("% ",t->comments, "");
+	print_comments("% ", t->comments, "");
 
 	set_fillcolor(t->color, NUMSHADES - 1);
 
@@ -1803,7 +1812,6 @@ gentikz_text(F_text *t)
 	    }
 	} else for (cp = (unsigned char*)t->cstring; *cp; ++cp) {
 #ifdef I18N
-	    extern bool support_i18n;
 	    if (support_i18n && (t->font <= 2))
 		fputc(*cp, tfp);
 	    else
@@ -1823,7 +1831,7 @@ gentikz_text(F_text *t)
 	    } else
 		fputc(*cp, tfp);
 	}
-	fputs("};\n", tfp);
+	fputs("}\n", tfp);
 }
 
 void
@@ -1837,10 +1845,10 @@ gentikz_arc(F_arc *a)
 	    fprintf(tfp, "%%\n%% Fig ARC object\n%%\n");
 
 	/* print any comments prefixed with "%" */
-	print_comments("% ",a->comments, "");
+	print_comments("% ", a->comments, "");
 
-	/* nothing to do; return; */
-	if (NOTEXISTS(a) && !a->for_arrow && !a->back_arrow)
+	if (a->thickness <= 0 && a->fill_style == UNFILLED &&
+			!a->for_arrow && !a->back_arrow)
 	    return;
 
 	/* compute the angles in tikz-coordinate system */
